from utils import *

def inject(payload_name, file_name, straddr):
  print "{}[*]{} Starting injection into binary...{}".format(Bcolors.YELLOW, Bcolors.BOLD, Bcolors.ENDC)
  print
  addr = parse_int(straddr)

  payload = open(payload_name, "rb").read()
  vic = open(file_name, "rb").read()

  res = vic[:addr] + payload + vic[addr + len(payload):]

  f = open("{}.mod".format(file_name), "w")
  f.write(res)
  f.close()

  print "{}[*]{} Injection finished.{}".format(Bcolors.YELLOW, Bcolors.BOLD, Bcolors.ENDC)



# -*- coding: utf-8 -*-
"""Search for code cave in all binaries
Usage:
  cave_miner search [--size=<size>] <file_name>
  cave_miner inject <payload> <file_name> <address>

Options:
  -h --help      Show this help
  --version      Show the program version
  --size=<size>  The minimum size of the cave in bytes [default: 256]
"""
from docopt import docopt
import os.path
from cave_miner import *
import re

def print_banner():
  print """
    {gy}/========\{e}
   {gy}/{e}    {gn}||{e}    {gy}\{e}
        {gn}||{e}
        {gn}||{e}
        {gn}||{e}
   CAVE {gn}||{e} MINER
  """.format(gy=Bcolors.GREY, gn=Bcolors.GREEN, e=Bcolors.ENDC)

def test_file(filename):
  res = os.path.isfile(filename)

  if not res: print "{r}*** File {filename} doesnt exist ***{e}".format(filename=filename, r=Bcolors.RED, e=Bcolors.ENDC)
  return res

def test_number(number):
  pattern = re.compile("0[xX][0-9a-fA-F]+|\d+")
  res = pattern.match(number)

  if not res: print "{r}*** Number {number} not valid ***{e}".format(number=number, r=Bcolors.RED, e=Bcolors.ENDC)
  return res

def main():
  print_banner()
  args = docopt(__doc__, version='0.1')
  CONTINUE = True

  if args["search"] == True:
    CONTINUE = CONTINUE and test_file(args["<file_name>"])
    CONTINUE = CONTINUE and test_number(args["--size"])

    if CONTINUE: search(args["<file_name>"], args["--size"])

  elif args["inject"] == True:
    CONTINUE = CONTINUE and test_file(args["<payload>"])
    CONTINUE = CONTINUE and test_file(args["<file_name>"])
    CONTINUE = CONTINUE and test_number(args["<address>"])

    if CONTINUE: inject(args["<payload>"], args["<file_name>"], args["<address>"])



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import array
import struct
import zlib
from enum import Enum
from pkg_resources import parse_version

from kaitaistruct import __version__ as ks_version, KaitaiStruct, KaitaiStream, BytesIO


if parse_version(ks_version) < parse_version('0.7'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.7 or later is required, but you have %s" % (ks_version))

class MicrosoftPe(KaitaiStruct):

    class PeFormat(Enum):
        rom_image = 263
        pe32 = 267
        pe32_plus = 523
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self.mz1 = self._root.MzPlaceholder(self._io, self, self._root)
        self.mz2 = self._io.read_bytes((self.mz1.header_size - 64))
        self.pe_signature = self._io.ensure_fixed_contents(struct.pack('4b', 80, 69, 0, 0))
        self.coff_hdr = self._root.CoffHeader(self._io, self, self._root)
        self._raw_optional_hdr = self._io.read_bytes(self.coff_hdr.size_of_optional_header)
        io = KaitaiStream(BytesIO(self._raw_optional_hdr))
        self.optional_hdr = self._root.OptionalHeader(io, self, self._root)
        self.sections = [None] * (self.coff_hdr.number_of_sections)
        for i in range(self.coff_hdr.number_of_sections):
            self.sections[i] = self._root.Section(self._io, self, self._root)


    class OptionalHeaderWindows(KaitaiStruct):

        class SubsystemEnum(Enum):
            unknown = 0
            native = 1
            windows_gui = 2
            windows_cui = 3
            posix_cui = 7
            windows_ce_gui = 9
            efi_application = 10
            efi_boot_service_driver = 11
            efi_runtime_driver = 12
            efi_rom = 13
            xbox = 14
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            if self._parent.std.format == self._root.PeFormat.pe32:
                self.image_base_32 = self._io.read_u4le()

            if self._parent.std.format == self._root.PeFormat.pe32_plus:
                self.image_base_64 = self._io.read_u8le()

            self.section_alignment = self._io.read_u4le()
            self.file_alignment = self._io.read_u4le()
            self.major_operating_system_version = self._io.read_u2le()
            self.minor_operating_system_version = self._io.read_u2le()
            self.major_image_version = self._io.read_u2le()
            self.minor_image_version = self._io.read_u2le()
            self.major_subsystem_version = self._io.read_u2le()
            self.minor_subsystem_version = self._io.read_u2le()
            self.win32_version_value = self._io.read_u4le()
            self.size_of_image = self._io.read_u4le()
            self.size_of_headers = self._io.read_u4le()
            self.check_sum = self._io.read_u4le()
            self.subsystem = self._root.OptionalHeaderWindows.SubsystemEnum(self._io.read_u2le())
            self.dll_characteristics = self._io.read_u2le()
            if self._parent.std.format == self._root.PeFormat.pe32:
                self.size_of_stack_reserve_32 = self._io.read_u4le()

            if self._parent.std.format == self._root.PeFormat.pe32_plus:
                self.size_of_stack_reserve_64 = self._io.read_u8le()

            if self._parent.std.format == self._root.PeFormat.pe32:
                self.size_of_stack_commit_32 = self._io.read_u4le()

            if self._parent.std.format == self._root.PeFormat.pe32_plus:
                self.size_of_stack_commit_64 = self._io.read_u8le()

            if self._parent.std.format == self._root.PeFormat.pe32:
                self.size_of_heap_reserve_32 = self._io.read_u4le()

            if self._parent.std.format == self._root.PeFormat.pe32_plus:
                self.size_of_heap_reserve_64 = self._io.read_u8le()

            if self._parent.std.format == self._root.PeFormat.pe32:
                self.size_of_heap_commit_32 = self._io.read_u4le()

            if self._parent.std.format == self._root.PeFormat.pe32_plus:
                self.size_of_heap_commit_64 = self._io.read_u8le()

            self.loader_flags = self._io.read_u4le()
            self.number_of_rva_and_sizes = self._io.read_u4le()


    class OptionalHeaderDataDirs(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.export_table = self._root.DataDir(self._io, self, self._root)
            self.import_table = self._root.DataDir(self._io, self, self._root)
            self.resource_table = self._root.DataDir(self._io, self, self._root)
            self.exception_table = self._root.DataDir(self._io, self, self._root)
            self.certificate_table = self._root.DataDir(self._io, self, self._root)
            self.base_relocation_table = self._root.DataDir(self._io, self, self._root)
            self.debug = self._root.DataDir(self._io, self, self._root)
            self.architecture = self._root.DataDir(self._io, self, self._root)
            self.global_ptr = self._root.DataDir(self._io, self, self._root)
            self.tls_table = self._root.DataDir(self._io, self, self._root)
            self.load_config_table = self._root.DataDir(self._io, self, self._root)
            self.bound_import = self._root.DataDir(self._io, self, self._root)
            self.iat = self._root.DataDir(self._io, self, self._root)
            self.delay_import_descriptor = self._root.DataDir(self._io, self, self._root)
            self.clr_runtime_header = self._root.DataDir(self._io, self, self._root)


    class DataDir(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.virtual_address = self._io.read_u4le()
            self.size = self._io.read_u4le()


    class OptionalHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.std = self._root.OptionalHeaderStd(self._io, self, self._root)
            self.windows = self._root.OptionalHeaderWindows(self._io, self, self._root)
            self.data_dirs = self._root.OptionalHeaderDataDirs(self._io, self, self._root)


    class Section(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.name = (KaitaiStream.bytes_strip_right(self._io.read_bytes(8), 0)).decode(u"UTF-8")
            self.virtual_size = self._io.read_u4le()
            self.virtual_address = self._io.read_u4le()
            self.size_of_raw_data = self._io.read_u4le()
            self.pointer_to_raw_data = self._io.read_u4le()
            self.pointer_to_relocations = self._io.read_u4le()
            self.pointer_to_linenumbers = self._io.read_u4le()
            self.number_of_relocations = self._io.read_u2le()
            self.number_of_linenumbers = self._io.read_u2le()
            self.characteristics = self._io.read_u4le()

        @property
        def body(self):
            if hasattr(self, '_m_body'):
                return self._m_body if hasattr(self, '_m_body') else None

            _pos = self._io.pos()
            self._io.seek(self.pointer_to_raw_data)
            self._m_body = self._io.read_bytes(self.size_of_raw_data)
            self._io.seek(_pos)
            return self._m_body if hasattr(self, '_m_body') else None


    class MzPlaceholder(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.magic = self._io.ensure_fixed_contents(struct.pack('2b', 77, 90))
            self.data1 = self._io.read_bytes(58)
            self.header_size = self._io.read_u4le()


    class OptionalHeaderStd(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.format = self._root.PeFormat(self._io.read_u2le())
            self.major_linker_version = self._io.read_u1()
            self.minor_linker_version = self._io.read_u1()
            self.size_of_code = self._io.read_u4le()
            self.size_of_initialized_data = self._io.read_u4le()
            self.size_of_uninitialized_data = self._io.read_u4le()
            self.address_of_entry_point = self._io.read_u4le()
            self.base_of_code = self._io.read_u4le()
            if self.format == self._root.PeFormat.pe32:
                self.base_of_data = self._io.read_u4le()



    class CoffHeader(KaitaiStruct):

        class MachineType(Enum):
            unknown = 0
            i386 = 332
            r4000 = 358
            wcemipsv2 = 361
            sh3 = 418
            sh3dsp = 419
            sh4 = 422
            sh5 = 424
            arm = 448
            thumb = 450
            armnt = 452
            am33 = 467
            powerpc = 496
            powerpcfp = 497
            ia64 = 512
            mips16 = 614
            mipsfpu = 870
            mipsfpu16 = 1126
            ebc = 3772
            riscv32 = 20530
            riscv64 = 20580
            riscv128 = 20776
            amd64 = 34404
            m32r = 36929
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.machine = self._root.CoffHeader.MachineType(self._io.read_u2le())
            self.number_of_sections = self._io.read_u2le()
            self.time_date_stamp = self._io.read_u4le()
            self.pointer_to_symbol_table = self._io.read_u4le()
            self.number_of_symbols = self._io.read_u4le()
            self.size_of_optional_header = self._io.read_u2le()
            self.characteristics = self._io.read_u2le()






def parse_int(st):
  if "x" in st:
    return int(st, 16)
  elif "X" in st:
    return int(st, 16)
  else:
    return int(st)

class Bcolors:
  GREEN = '\033[92m'
  YELLOW = '\033[33m'
  ENDC = '\033[0m'
  GREY = '\033[37m'
  BOLD = '\033[1m'
  RED = '\033[31m'



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

import array
import struct
import zlib
from enum import Enum
from pkg_resources import parse_version

from kaitaistruct import __version__ as ks_version, KaitaiStruct, KaitaiStream, BytesIO


if parse_version(ks_version) < parse_version('0.7'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.7 or later is required, but you have %s" % (ks_version))

class MachO(KaitaiStruct):

    class LoadCommandType(Enum):
        segment = 1
        symtab = 2
        symseg = 3
        thread = 4
        unix_thread = 5
        load_fvm_lib = 6
        id_fvm_lib = 7
        ident = 8
        fvm_file = 9
        prepage = 10
        dysymtab = 11
        load_dylib = 12
        id_dylib = 13
        load_dylinker = 14
        id_dylinker = 15
        prebound_dylib = 16
        routines = 17
        sub_framework = 18
        sub_umbrella = 19
        sub_client = 20
        sub_library = 21
        twolevel_hints = 22
        prebind_cksum = 23
        segment_64 = 25
        routines_64 = 26
        uuid = 27
        code_signature = 29
        segment_split_info = 30
        lazy_load_dylib = 32
        encryption_info = 33
        dyld_info = 34
        load_upward_dylib = 35
        version_min_macosx = 36
        version_min_iphoneos = 37
        function_starts = 38
        dyld_environment = 39
        main = 40
        data_in_code = 41
        source_version = 42
        dylib_code_sign_drs = 43
        encryption_info_64 = 44
        linker_option = 45
        linker_optimization_hint = 46
        version_min_tvos = 47
        version_min_watchos = 48
        req_dyld = 2147483648
        load_weak_dylib = 2147483672
        rpath = 2147483676
        reexport_dylib = 2147483679
        dyld_info_only = 2147483682
        main2 = 2147483688

    class MachoFlags(Enum):
        no_undefs = 1
        incr_link = 2
        dyld_link = 4
        bind_at_load = 8
        prebound = 16
        split_segs = 32
        lazy_init = 64
        two_level = 128
        force_flat = 256
        no_multi_defs = 512
        no_fix_prebinding = 1024
        prebindable = 2048
        all_mods_bound = 4096
        subsections_via_symbols = 8192
        canonical = 16384
        weak_defines = 32768
        binds_to_weak = 65536
        allow_stack_execution = 131072
        root_safe = 262144
        setuid_safe = 524288
        no_reexported_dylibs = 1048576
        pie = 2097152
        dead_strippable_dylib = 4194304
        has_tlv_descriptors = 8388608
        no_heap_execution = 16777216
        app_extension_safe = 33554432

    class MagicType(Enum):
        fat_le = 3199925962
        fat_be = 3405691582
        macho_le_x86 = 3472551422
        macho_le_x64 = 3489328638
        macho_be_x86 = 4277009102
        macho_be_x64 = 4277009103

    class FileType(Enum):
        object = 1
        execute = 2
        fvmlib = 3
        core = 4
        preload = 5
        dylib = 6
        dylinker = 7
        bundle = 8
        dylib_stub = 9
        dsym = 10
        kext_bundle = 11

    class CpuType(Enum):
        vax = 1
        romp = 2
        ns32032 = 4
        ns32332 = 5
        i386 = 7
        mips = 8
        ns32532 = 9
        hppa = 11
        arm = 12
        mc88000 = 13
        sparc = 14
        i860 = 15
        i860_little = 16
        rs6000 = 17
        powerpc = 18
        abi64 = 16777216
        x86_64 = 16777223
        powerpc64 = 16777234
        any = 4294967295
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self.magic = self._root.MagicType(self._io.read_u4be())
        self.header = self._root.MachHeader(self._io, self, self._root)
        self.load_commands = [None] * (self.header.ncmds)
        for i in range(self.header.ncmds):
            self.load_commands[i] = self._root.LoadCommand(self._io, self, self._root)


    class RpathCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.path_offset = self._io.read_u4le()
            self.path = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf-8")


    class Uleb128(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.b1 = self._io.read_u1()
            if (self.b1 & 128) != 0:
                self.b2 = self._io.read_u1()

            if (self.b2 & 128) != 0:
                self.b3 = self._io.read_u1()

            if (self.b3 & 128) != 0:
                self.b4 = self._io.read_u1()

            if (self.b4 & 128) != 0:
                self.b5 = self._io.read_u1()

            if (self.b5 & 128) != 0:
                self.b6 = self._io.read_u1()

            if (self.b6 & 128) != 0:
                self.b7 = self._io.read_u1()

            if (self.b7 & 128) != 0:
                self.b8 = self._io.read_u1()

            if (self.b8 & 128) != 0:
                self.b9 = self._io.read_u1()

            if (self.b9 & 128) != 0:
                self.b10 = self._io.read_u1()


        @property
        def value(self):
            if hasattr(self, '_m_value'):
                return self._m_value if hasattr(self, '_m_value') else None

            self._m_value = (((self.b1 % 128) << 0) + (0 if (self.b1 & 128) == 0 else (((self.b2 % 128) << 7) + (0 if (self.b2 & 128) == 0 else (((self.b3 % 128) << 14) + (0 if (self.b3 & 128) == 0 else (((self.b4 % 128) << 21) + (0 if (self.b4 & 128) == 0 else (((self.b5 % 128) << 28) + (0 if (self.b5 & 128) == 0 else (((self.b6 % 128) << 35) + (0 if (self.b6 & 128) == 0 else (((self.b7 % 128) << 42) + (0 if (self.b7 & 128) == 0 else (((self.b8 % 128) << 49) + (0 if (self.b8 & 128) == 0 else (((self.b9 % 128) << 56) + (0 if (self.b8 & 128) == 0 else ((self.b10 % 128) << 63)))))))))))))))))))
            return self._m_value if hasattr(self, '_m_value') else None


    class SourceVersionCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.version = self._io.read_u8le()


    class CsBlob(KaitaiStruct):

        class CsMagic(Enum):
            blob_wrapper = 4208855809
            requirement = 4208856064
            requirements = 4208856065
            code_directory = 4208856066
            embedded_signature = 4208856256
            detached_signature = 4208856257
            entitlement = 4208882033
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.magic = self._root.CsBlob.CsMagic(self._io.read_u4be())
            self.length = self._io.read_u4be()
            _on = self.magic
            if _on == self._root.CsBlob.CsMagic.detached_signature:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.SuperBlob(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.embedded_signature:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.SuperBlob(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.entitlement:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.Entitlement(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.blob_wrapper:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.BlobWrapper(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.requirement:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.Requirement(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.code_directory:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.CodeDirectory(io, self, self._root)
            elif _on == self._root.CsBlob.CsMagic.requirements:
                self._raw_body = self._io.read_bytes((self.length - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CsBlob.Entitlements(io, self, self._root)
            else:
                self.body = self._io.read_bytes((self.length - 8))

        class Entitlement(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.data = self._io.read_bytes_full()


        class CodeDirectory(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.version = self._io.read_u4be()
                self.flags = self._io.read_u4be()
                self.hash_offset = self._io.read_u4be()
                self.ident_offset = self._io.read_u4be()
                self.n_special_slots = self._io.read_u4be()
                self.n_code_slots = self._io.read_u4be()
                self.code_limit = self._io.read_u4be()
                self.hash_size = self._io.read_u1()
                self.hash_type = self._io.read_u1()
                self.spare1 = self._io.read_u1()
                self.page_size = self._io.read_u1()
                self.spare2 = self._io.read_u4be()
                if self.version >= 131328:
                    self.scatter_offset = self._io.read_u4be()

                if self.version >= 131584:
                    self.team_id_offset = self._io.read_u4be()


            @property
            def ident(self):
                if hasattr(self, '_m_ident'):
                    return self._m_ident if hasattr(self, '_m_ident') else None

                _pos = self._io.pos()
                self._io.seek((self.ident_offset - 8))
                self._m_ident = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf-8")
                self._io.seek(_pos)
                return self._m_ident if hasattr(self, '_m_ident') else None

            @property
            def team_id(self):
                if hasattr(self, '_m_team_id'):
                    return self._m_team_id if hasattr(self, '_m_team_id') else None

                _pos = self._io.pos()
                self._io.seek((self.team_id_offset - 8))
                self._m_team_id = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf-8")
                self._io.seek(_pos)
                return self._m_team_id if hasattr(self, '_m_team_id') else None

            @property
            def hashes(self):
                if hasattr(self, '_m_hashes'):
                    return self._m_hashes if hasattr(self, '_m_hashes') else None

                _pos = self._io.pos()
                self._io.seek(((self.hash_offset - 8) - (self.hash_size * self.n_special_slots)))
                self._m_hashes = [None] * ((self.n_special_slots + self.n_code_slots))
                for i in range((self.n_special_slots + self.n_code_slots)):
                    self._m_hashes[i] = self._io.read_bytes(self.hash_size)

                self._io.seek(_pos)
                return self._m_hashes if hasattr(self, '_m_hashes') else None


        class EntitlementsBlobIndex(KaitaiStruct):

            class RequirementType(Enum):
                host = 1
                guest = 2
                designated = 3
                library = 4
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.type = self._root.CsBlob.EntitlementsBlobIndex.RequirementType(self._io.read_u4be())
                self.offset = self._io.read_u4be()

            @property
            def value(self):
                if hasattr(self, '_m_value'):
                    return self._m_value if hasattr(self, '_m_value') else None

                _pos = self._io.pos()
                self._io.seek((self.offset - 8))
                self._m_value = self._root.CsBlob(self._io, self, self._root)
                self._io.seek(_pos)
                return self._m_value if hasattr(self, '_m_value') else None


        class Data(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.length = self._io.read_u4be()
                self.value = self._io.read_bytes(self.length)
                self.padding = self._io.read_bytes((4 - (self.length & 3)))


        class SuperBlob(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.count = self._io.read_u4be()
                self.blobs = [None] * (self.count)
                for i in range(self.count):
                    self.blobs[i] = self._root.CsBlob.BlobIndex(self._io, self, self._root)



        class Expr(KaitaiStruct):

            class OpEnum(Enum):
                false = 0
                true = 1
                ident = 2
                apple_anchor = 3
                anchor_hash = 4
                info_key_value = 5
                and_op = 6
                or_op = 7
                cd_hash = 8
                not_op = 9
                info_key_field = 10
                cert_field = 11
                trusted_cert = 12
                trusted_certs = 13
                cert_generic = 14
                apple_generic_anchor = 15
                entitlement_field = 16

            class CertSlot(Enum):
                left_cert = 0
                anchor_cert = 4294967295
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.op = self._root.CsBlob.Expr.OpEnum(self._io.read_u4be())
                _on = self.op
                if _on == self._root.CsBlob.Expr.OpEnum.cert_generic:
                    self.data = self._root.CsBlob.Expr.CertGenericExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.apple_generic_anchor:
                    self.data = self._root.CsBlob.Expr.AppleGenericAnchorExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.info_key_field:
                    self.data = self._root.CsBlob.Expr.InfoKeyFieldExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.and_op:
                    self.data = self._root.CsBlob.Expr.AndExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.anchor_hash:
                    self.data = self._root.CsBlob.Expr.AnchorHashExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.info_key_value:
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.or_op:
                    self.data = self._root.CsBlob.Expr.OrExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.trusted_cert:
                    self.data = self._root.CsBlob.Expr.CertSlotExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.not_op:
                    self.data = self._root.CsBlob.Expr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.ident:
                    self.data = self._root.CsBlob.Expr.IdentExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.cert_field:
                    self.data = self._root.CsBlob.Expr.CertFieldExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.entitlement_field:
                    self.data = self._root.CsBlob.Expr.EntitlementFieldExpr(self._io, self, self._root)
                elif _on == self._root.CsBlob.Expr.OpEnum.cd_hash:
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)

            class InfoKeyFieldExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)
                    self.match = self._root.CsBlob.Match(self._io, self, self._root)


            class CertSlotExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.value = self._root.CsBlob.Expr.CertSlot(self._io.read_u4be())


            class CertGenericExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.cert_slot = self._root.CsBlob.Expr.CertSlot(self._io.read_u4be())
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)
                    self.match = self._root.CsBlob.Match(self._io, self, self._root)


            class IdentExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.identifier = self._root.CsBlob.Data(self._io, self, self._root)


            class CertFieldExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.cert_slot = self._root.CsBlob.Expr.CertSlot(self._io.read_u4be())
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)
                    self.match = self._root.CsBlob.Match(self._io, self, self._root)


            class AnchorHashExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.cert_slot = self._root.CsBlob.Expr.CertSlot(self._io.read_u4be())
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)


            class AppleGenericAnchorExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self

                @property
                def value(self):
                    if hasattr(self, '_m_value'):
                        return self._m_value if hasattr(self, '_m_value') else None

                    self._m_value = u"anchor apple generic"
                    return self._m_value if hasattr(self, '_m_value') else None


            class EntitlementFieldExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)
                    self.match = self._root.CsBlob.Match(self._io, self, self._root)


            class AndExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.left = self._root.CsBlob.Expr(self._io, self, self._root)
                    self.right = self._root.CsBlob.Expr(self._io, self, self._root)


            class OrExpr(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.left = self._root.CsBlob.Expr(self._io, self, self._root)
                    self.right = self._root.CsBlob.Expr(self._io, self, self._root)



        class BlobIndex(KaitaiStruct):

            class CsslotType(Enum):
                code_directory = 0
                info_slot = 1
                requirements = 2
                resource_dir = 3
                application = 4
                entitlements = 5
                alternate_code_directories = 4096
                signature_slot = 65536
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.type = self._root.CsBlob.BlobIndex.CsslotType(self._io.read_u4be())
                self.offset = self._io.read_u4be()

            @property
            def blob(self):
                if hasattr(self, '_m_blob'):
                    return self._m_blob if hasattr(self, '_m_blob') else None

                io = self._parent._io
                _pos = io.pos()
                io.seek((self.offset - 8))
                self._raw__m_blob = io.read_bytes_full()
                io = KaitaiStream(BytesIO(self._raw__m_blob))
                self._m_blob = self._root.CsBlob(io, self, self._root)
                io.seek(_pos)
                return self._m_blob if hasattr(self, '_m_blob') else None


        class Match(KaitaiStruct):

            class Op(Enum):
                exists = 0
                equal = 1
                contains = 2
                begins_with = 3
                ends_with = 4
                less_than = 5
                greater_than = 6
                less_equal = 7
                greater_equal = 8
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.match_op = self._root.CsBlob.Match.Op(self._io.read_u4be())
                if self.match_op != self._root.CsBlob.Match.Op.exists:
                    self.data = self._root.CsBlob.Data(self._io, self, self._root)



        class Requirement(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.kind = self._io.read_u4be()
                self.expr = self._root.CsBlob.Expr(self._io, self, self._root)


        class BlobWrapper(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.data = self._io.read_bytes_full()


        class Entitlements(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.count = self._io.read_u4be()
                self.items = [None] * (self.count)
                for i in range(self.count):
                    self.items[i] = self._root.CsBlob.EntitlementsBlobIndex(self._io, self, self._root)




    class SegmentCommand64(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.segname = (KaitaiStream.bytes_strip_right(self._io.read_bytes(16), 0)).decode(u"ascii")
            self.vmaddr = self._io.read_u8le()
            self.vmsize = self._io.read_u8le()
            self.fileoff = self._io.read_u8le()
            self.filesize = self._io.read_u8le()
            self.maxprot = self._io.read_u4le()
            self.initprot = self._io.read_u4le()
            self.nsects = self._io.read_u4le()
            self.flags = self._io.read_u4le()
            self.sections = [None] * (self.nsects)
            for i in range(self.nsects):
                self.sections[i] = self._root.SegmentCommand64.Section64(self._io, self, self._root)


        class Section64(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.sect_name = (KaitaiStream.bytes_strip_right(self._io.read_bytes(16), 0)).decode(u"ascii")
                self.seg_name = (KaitaiStream.bytes_strip_right(self._io.read_bytes(16), 0)).decode(u"ascii")
                self.addr = self._io.read_u8le()
                self.size = self._io.read_u8le()
                self.offset = self._io.read_u4le()
                self.align = self._io.read_u4le()
                self.reloff = self._io.read_u4le()
                self.nreloc = self._io.read_u4le()
                self.flags = self._io.read_u4le()
                self.reserved1 = self._io.read_u4le()
                self.reserved2 = self._io.read_u4le()
                self.reserved3 = self._io.read_u4le()

            class CfStringList(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.items = []
                    while not self._io.is_eof():
                        self.items.append(self._root.SegmentCommand64.Section64.CfString(self._io, self, self._root))



            class CfString(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.isa = self._io.read_u8le()
                    self.info = self._io.read_u8le()
                    self.data = self._io.read_u8le()
                    self.length = self._io.read_u8le()


            class EhFrameItem(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.length = self._io.read_u4le()
                    if self.length == 4294967295:
                        self.length64 = self._io.read_u8le()

                    self.id = self._io.read_u4le()
                    if self.length > 0:
                        _on = self.id
                        if _on == 0:
                            self._raw_body = self._io.read_bytes((self.length - 4))
                            io = KaitaiStream(BytesIO(self._raw_body))
                            self.body = self._root.SegmentCommand64.Section64.EhFrameItem.Cie(io, self, self._root)
                        else:
                            self.body = self._io.read_bytes((self.length - 4))


                class CharChain(KaitaiStruct):
                    def __init__(self, _io, _parent=None, _root=None):
                        self._io = _io
                        self._parent = _parent
                        self._root = _root if _root else self
                        self.chr = self._io.read_u1()
                        if self.chr != 0:
                            self.next = self._root.SegmentCommand64.Section64.EhFrameItem.CharChain(self._io, self, self._root)



                class Cie(KaitaiStruct):
                    def __init__(self, _io, _parent=None, _root=None):
                        self._io = _io
                        self._parent = _parent
                        self._root = _root if _root else self
                        self.version = self._io.read_u1()
                        self.aug_str = self._root.SegmentCommand64.Section64.EhFrameItem.CharChain(self._io, self, self._root)
                        self.code_alignment_factor = self._root.Uleb128(self._io, self, self._root)
                        self.data_alignment_factor = self._root.Uleb128(self._io, self, self._root)
                        self.return_address_register = self._io.read_u1()
                        if self.aug_str.chr == 122:
                            self.augmentation = self._root.SegmentCommand64.Section64.EhFrameItem.AugmentationEntry(self._io, self, self._root)



                class AugmentationEntry(KaitaiStruct):
                    def __init__(self, _io, _parent=None, _root=None):
                        self._io = _io
                        self._parent = _parent
                        self._root = _root if _root else self
                        self.length = self._root.Uleb128(self._io, self, self._root)
                        if self._parent.aug_str.next.chr == 82:
                            self.fde_pointer_encoding = self._io.read_u1()




            class EhFrame(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.items = []
                    while not self._io.is_eof():
                        self.items.append(self._root.SegmentCommand64.Section64.EhFrameItem(self._io, self, self._root))



            class PointerList(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.items = []
                    while not self._io.is_eof():
                        self.items.append(self._io.read_u8le())



            class StringList(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.strings = []
                    while not self._io.is_eof():
                        self.strings.append((self._io.read_bytes_term(0, False, True, True)).decode(u"ascii"))



            @property
            def data(self):
                if hasattr(self, '_m_data'):
                    return self._m_data if hasattr(self, '_m_data') else None

                io = self._root._io
                _pos = io.pos()
                io.seek(self.offset)
                _on = self.sect_name
                if _on == u"__objc_nlclslist":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_methname":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.StringList(io, self, self._root)
                elif _on == u"__nl_symbol_ptr":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__la_symbol_ptr":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_selrefs":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__cstring":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.StringList(io, self, self._root)
                elif _on == u"__objc_classlist":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_protolist":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_imageinfo":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_methtype":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.StringList(io, self, self._root)
                elif _on == u"__cfstring":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.CfStringList(io, self, self._root)
                elif _on == u"__objc_classrefs":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_protorefs":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__objc_classname":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.StringList(io, self, self._root)
                elif _on == u"__got":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                elif _on == u"__eh_frame":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.EhFrame(io, self, self._root)
                elif _on == u"__objc_superrefs":
                    self._raw__m_data = io.read_bytes(self.size)
                    io = KaitaiStream(BytesIO(self._raw__m_data))
                    self._m_data = self._root.SegmentCommand64.Section64.PointerList(io, self, self._root)
                else:
                    self._m_data = io.read_bytes(self.size)
                io.seek(_pos)
                return self._m_data if hasattr(self, '_m_data') else None



    class DysymtabCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.i_local_sym = self._io.read_u4le()
            self.n_local_sym = self._io.read_u4le()
            self.i_ext_def_sym = self._io.read_u4le()
            self.n_ext_def_sym = self._io.read_u4le()
            self.i_undef_sym = self._io.read_u4le()
            self.n_undef_sym = self._io.read_u4le()
            self.toc_off = self._io.read_u4le()
            self.n_toc = self._io.read_u4le()
            self.mod_tab_off = self._io.read_u4le()
            self.n_mod_tab = self._io.read_u4le()
            self.ext_ref_sym_off = self._io.read_u4le()
            self.n_ext_ref_syms = self._io.read_u4le()
            self.indirect_sym_off = self._io.read_u4le()
            self.n_indirect_syms = self._io.read_u4le()
            self.ext_rel_off = self._io.read_u4le()
            self.n_ext_rel = self._io.read_u4le()
            self.loc_rel_off = self._io.read_u4le()
            self.n_loc_rel = self._io.read_u4le()

        @property
        def indirect_symbols(self):
            if hasattr(self, '_m_indirect_symbols'):
                return self._m_indirect_symbols if hasattr(self, '_m_indirect_symbols') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.indirect_sym_off)
            self._m_indirect_symbols = [None] * (self.n_indirect_syms)
            for i in range(self.n_indirect_syms):
                self._m_indirect_symbols[i] = io.read_u4le()

            io.seek(_pos)
            return self._m_indirect_symbols if hasattr(self, '_m_indirect_symbols') else None


    class MachHeader(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.cputype = self._root.CpuType(self._io.read_u4le())
            self.cpusubtype = self._io.read_u4le()
            self.filetype = self._root.FileType(self._io.read_u4le())
            self.ncmds = self._io.read_u4le()
            self.sizeofcmds = self._io.read_u4le()
            self.flags = self._io.read_u4le()
            if  ((self._root.magic == self._root.MagicType.macho_be_x64) or (self._root.magic == self._root.MagicType.macho_le_x64)) :
                self.reserved = self._io.read_u4le()



    class LinkeditDataCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.data_off = self._io.read_u4le()
            self.data_size = self._io.read_u4le()


    class Version(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.p1 = self._io.read_u1()
            self.minor = self._io.read_u1()
            self.major = self._io.read_u1()
            self.release = self._io.read_u1()


    class CodeSignatureCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.data_off = self._io.read_u4le()
            self.data_size = self._io.read_u4le()

        @property
        def code_signature(self):
            if hasattr(self, '_m_code_signature'):
                return self._m_code_signature if hasattr(self, '_m_code_signature') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.data_off)
            self._raw__m_code_signature = io.read_bytes(self.data_size)
            io = KaitaiStream(BytesIO(self._raw__m_code_signature))
            self._m_code_signature = self._root.CsBlob(io, self, self._root)
            io.seek(_pos)
            return self._m_code_signature if hasattr(self, '_m_code_signature') else None


    class DyldInfoCommand(KaitaiStruct):

        class BindOpcode(Enum):
            done = 0
            set_dylib_ordinal_immediate = 16
            set_dylib_ordinal_uleb = 32
            set_dylib_special_immediate = 48
            set_symbol_trailing_flags_immediate = 64
            set_type_immediate = 80
            set_append_sleb = 96
            set_segment_and_offset_uleb = 112
            add_address_uleb = 128
            do_bind = 144
            do_bind_add_address_uleb = 160
            do_bind_add_address_immediate_scaled = 176
            do_bind_uleb_times_skipping_uleb = 192
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.rebase_off = self._io.read_u4le()
            self.rebase_size = self._io.read_u4le()
            self.bind_off = self._io.read_u4le()
            self.bind_size = self._io.read_u4le()
            self.weak_bind_off = self._io.read_u4le()
            self.weak_bind_size = self._io.read_u4le()
            self.lazy_bind_off = self._io.read_u4le()
            self.lazy_bind_size = self._io.read_u4le()
            self.export_off = self._io.read_u4le()
            self.export_size = self._io.read_u4le()

        class BindItem(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.opcode_and_immediate = self._io.read_u1()
                if  ((self.opcode == self._root.DyldInfoCommand.BindOpcode.set_dylib_ordinal_uleb) or (self.opcode == self._root.DyldInfoCommand.BindOpcode.set_append_sleb) or (self.opcode == self._root.DyldInfoCommand.BindOpcode.set_segment_and_offset_uleb) or (self.opcode == self._root.DyldInfoCommand.BindOpcode.add_address_uleb) or (self.opcode == self._root.DyldInfoCommand.BindOpcode.do_bind_add_address_uleb) or (self.opcode == self._root.DyldInfoCommand.BindOpcode.do_bind_uleb_times_skipping_uleb)) :
                    self.uleb = self._root.Uleb128(self._io, self, self._root)

                if self.opcode == self._root.DyldInfoCommand.BindOpcode.do_bind_uleb_times_skipping_uleb:
                    self.skip = self._root.Uleb128(self._io, self, self._root)

                if self.opcode == self._root.DyldInfoCommand.BindOpcode.set_symbol_trailing_flags_immediate:
                    self.symbol = (self._io.read_bytes_term(0, False, True, True)).decode(u"ascii")


            @property
            def opcode(self):
                if hasattr(self, '_m_opcode'):
                    return self._m_opcode if hasattr(self, '_m_opcode') else None

                self._m_opcode = self._root.DyldInfoCommand.BindOpcode((self.opcode_and_immediate & 240))
                return self._m_opcode if hasattr(self, '_m_opcode') else None

            @property
            def immediate(self):
                if hasattr(self, '_m_immediate'):
                    return self._m_immediate if hasattr(self, '_m_immediate') else None

                self._m_immediate = (self.opcode_and_immediate & 15)
                return self._m_immediate if hasattr(self, '_m_immediate') else None


        class RebaseData(KaitaiStruct):

            class Opcode(Enum):
                done = 0
                set_type_immediate = 16
                set_segment_and_offset_uleb = 32
                add_address_uleb = 48
                add_address_immediate_scaled = 64
                do_rebase_immediate_times = 80
                do_rebase_uleb_times = 96
                do_rebase_add_address_uleb = 112
                do_rebase_uleb_times_skipping_uleb = 128
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.items = []
                while True:
                    _ = self._root.DyldInfoCommand.RebaseData.RebaseItem(self._io, self, self._root)
                    self.items.append(_)
                    if _.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.done:
                        break

            class RebaseItem(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.opcode_and_immediate = self._io.read_u1()
                    if  ((self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.set_segment_and_offset_uleb) or (self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.add_address_uleb) or (self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.do_rebase_uleb_times) or (self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.do_rebase_add_address_uleb) or (self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.do_rebase_uleb_times_skipping_uleb)) :
                        self.uleb = self._root.Uleb128(self._io, self, self._root)

                    if self.opcode == self._root.DyldInfoCommand.RebaseData.Opcode.do_rebase_uleb_times_skipping_uleb:
                        self.skip = self._root.Uleb128(self._io, self, self._root)


                @property
                def opcode(self):
                    if hasattr(self, '_m_opcode'):
                        return self._m_opcode if hasattr(self, '_m_opcode') else None

                    self._m_opcode = self._root.DyldInfoCommand.RebaseData.Opcode((self.opcode_and_immediate & 240))
                    return self._m_opcode if hasattr(self, '_m_opcode') else None

                @property
                def immediate(self):
                    if hasattr(self, '_m_immediate'):
                        return self._m_immediate if hasattr(self, '_m_immediate') else None

                    self._m_immediate = (self.opcode_and_immediate & 15)
                    return self._m_immediate if hasattr(self, '_m_immediate') else None



        class ExportNode(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.terminal_size = self._root.Uleb128(self._io, self, self._root)
                self.children_count = self._io.read_u1()
                self.children = [None] * (self.children_count)
                for i in range(self.children_count):
                    self.children[i] = self._root.DyldInfoCommand.ExportNode.Child(self._io, self, self._root)

                self.terminal = self._io.read_bytes(self.terminal_size.value)

            class Child(KaitaiStruct):
                def __init__(self, _io, _parent=None, _root=None):
                    self._io = _io
                    self._parent = _parent
                    self._root = _root if _root else self
                    self.name = (self._io.read_bytes_term(0, False, True, True)).decode(u"ascii")
                    self.node_offset = self._root.Uleb128(self._io, self, self._root)

                @property
                def value(self):
                    if hasattr(self, '_m_value'):
                        return self._m_value if hasattr(self, '_m_value') else None

                    _pos = self._io.pos()
                    self._io.seek(self.node_offset.value)
                    self._m_value = self._root.DyldInfoCommand.ExportNode(self._io, self, self._root)
                    self._io.seek(_pos)
                    return self._m_value if hasattr(self, '_m_value') else None



        class BindData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.items = []
                while True:
                    _ = self._root.DyldInfoCommand.BindItem(self._io, self, self._root)
                    self.items.append(_)
                    if _.opcode == self._root.DyldInfoCommand.BindOpcode.done:
                        break


        class LazyBindData(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.items = []
                while not self._io.is_eof():
                    self.items.append(self._root.DyldInfoCommand.BindItem(self._io, self, self._root))



        @property
        def rebase(self):
            if hasattr(self, '_m_rebase'):
                return self._m_rebase if hasattr(self, '_m_rebase') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.rebase_off)
            self._raw__m_rebase = io.read_bytes(self.rebase_size)
            io = KaitaiStream(BytesIO(self._raw__m_rebase))
            self._m_rebase = self._root.DyldInfoCommand.RebaseData(io, self, self._root)
            io.seek(_pos)
            return self._m_rebase if hasattr(self, '_m_rebase') else None

        @property
        def bind(self):
            if hasattr(self, '_m_bind'):
                return self._m_bind if hasattr(self, '_m_bind') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.bind_off)
            self._raw__m_bind = io.read_bytes(self.bind_size)
            io = KaitaiStream(BytesIO(self._raw__m_bind))
            self._m_bind = self._root.DyldInfoCommand.BindData(io, self, self._root)
            io.seek(_pos)
            return self._m_bind if hasattr(self, '_m_bind') else None

        @property
        def lazy_bind(self):
            if hasattr(self, '_m_lazy_bind'):
                return self._m_lazy_bind if hasattr(self, '_m_lazy_bind') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.lazy_bind_off)
            self._raw__m_lazy_bind = io.read_bytes(self.lazy_bind_size)
            io = KaitaiStream(BytesIO(self._raw__m_lazy_bind))
            self._m_lazy_bind = self._root.DyldInfoCommand.LazyBindData(io, self, self._root)
            io.seek(_pos)
            return self._m_lazy_bind if hasattr(self, '_m_lazy_bind') else None

        @property
        def exports(self):
            if hasattr(self, '_m_exports'):
                return self._m_exports if hasattr(self, '_m_exports') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.export_off)
            self._raw__m_exports = io.read_bytes(self.export_size)
            io = KaitaiStream(BytesIO(self._raw__m_exports))
            self._m_exports = self._root.DyldInfoCommand.ExportNode(io, self, self._root)
            io.seek(_pos)
            return self._m_exports if hasattr(self, '_m_exports') else None


    class DylinkerCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.name = self._root.LcStr(self._io, self, self._root)


    class DylibCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.name_offset = self._io.read_u4le()
            self.timestamp = self._io.read_u4le()
            self.current_version = self._io.read_u4le()
            self.compatibility_version = self._io.read_u4le()
            self.name = (self._io.read_bytes_term(0, False, True, True)).decode(u"utf-8")


    class LcStr(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.length = self._io.read_u4le()
            self.value = (self._io.read_bytes_term(0, False, True, True)).decode(u"UTF-8")


    class LoadCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.type = self._root.LoadCommandType(self._io.read_u4le())
            self.size = self._io.read_u4le()
            _on = self.type
            if _on == self._root.LoadCommandType.rpath:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.RpathCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.source_version:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.SourceVersionCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.load_dylinker:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.DylinkerCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.symtab:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.SymtabCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.main:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.EntryPointCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.function_starts:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.LinkeditDataCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.version_min_macosx:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.VersionMinCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.data_in_code:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.LinkeditDataCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.uuid:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.UuidCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.dyld_info_only:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.DyldInfoCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.code_signature:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.CodeSignatureCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.dysymtab:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.DysymtabCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.load_dylib:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.DylibCommand(io, self, self._root)
            elif _on == self._root.LoadCommandType.segment_64:
                self._raw_body = self._io.read_bytes((self.size - 8))
                io = KaitaiStream(BytesIO(self._raw_body))
                self.body = self._root.SegmentCommand64(io, self, self._root)
            else:
                self.body = self._io.read_bytes((self.size - 8))


    class UuidCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.uuid = self._io.read_bytes(16)


    class SymtabCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.sym_off = self._io.read_u4le()
            self.n_syms = self._io.read_u4le()
            self.str_off = self._io.read_u4le()
            self.str_size = self._io.read_u4le()

        class StrTable(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.unknown = self._io.read_u4le()
                self.items = []
                while True:
                    _ = (self._io.read_bytes_term(0, False, True, True)).decode(u"ascii")
                    self.items.append(_)
                    if _ == u"":
                        break


        class Nlist64(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self.un = self._io.read_u4le()
                self.type = self._io.read_u1()
                self.sect = self._io.read_u1()
                self.desc = self._io.read_u2le()
                self.value = self._io.read_u8le()


        @property
        def symbols(self):
            if hasattr(self, '_m_symbols'):
                return self._m_symbols if hasattr(self, '_m_symbols') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.sym_off)
            self._m_symbols = [None] * (self.n_syms)
            for i in range(self.n_syms):
                self._m_symbols[i] = self._root.SymtabCommand.Nlist64(io, self, self._root)

            io.seek(_pos)
            return self._m_symbols if hasattr(self, '_m_symbols') else None

        @property
        def strs(self):
            if hasattr(self, '_m_strs'):
                return self._m_strs if hasattr(self, '_m_strs') else None

            io = self._root._io
            _pos = io.pos()
            io.seek(self.str_off)
            self._raw__m_strs = io.read_bytes(self.str_size)
            io = KaitaiStream(BytesIO(self._raw__m_strs))
            self._m_strs = self._root.SymtabCommand.StrTable(io, self, self._root)
            io.seek(_pos)
            return self._m_strs if hasattr(self, '_m_strs') else None


    class VersionMinCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.version = self._root.Version(self._io, self, self._root)
            self.reserved = self._root.Version(self._io, self, self._root)


    class EntryPointCommand(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            self.entry_off = self._io.read_u8le()
            self.stack_size = self._io.read_u8le()






from utils import *
from formats import * 
from struct import *

def search_cave(name, body, cave_size, file_offset, vaddr, infos=""):
  null_count = 0

  for offset in xrange(len(body)):
    byte = body[offset]

    if byte == "\x00":
      null_count += 1
    else:
      if null_count >= cave_size:
        print "{}[*]{} New cave detected !{}".format(Bcolors.YELLOW, Bcolors.BOLD, Bcolors.ENDC)
        print "  section_name: {}".format(name)
        print "  cave_begin:   0x{:08x}".format(file_offset + offset - null_count)
        print "  cave_end:     0x{:08x}".format(file_offset + offset)
        print "  cave_size:    0x{:08x}".format(null_count)
        print "  vaddress:     0x{:08x}".format(vaddr + offset - null_count)
        print "  infos:        {}".format(infos)
        print

      null_count = 0

def parse_macho_flags(byte):
  ret = []

  if 0x1 & byte == 0x1: ret.append("READ")
  if 0x2 & byte == 0x2: ret.append("WRITE")
  if 0x4 & byte == 0x4: ret.append("EXECUTE")

  return ", ".join(ret)

def parse_sh_flags(byte):
  ret = []

  if 0x1 & byte == 0x1: ret.append("SHF_WRITE")
  if 0x2 & byte == 0x2: ret.append("SHF_ALLOC")
  if 0x4 & byte == 0x4: ret.append("SHF_EXECINSTR")
  if 0x10 & byte == 0x20: ret.append("SHF_MERGE")
  if 0x20 & byte == 0x20: ret.append("SHF_STRINGS")
  if 0x40 & byte == 0x40: ret.append("SHF_INFO_LINK")
  if 0x80 & byte == 0x80: ret.append("SHF_LINK_ORDER")
  if 0x100 & byte == 0x100: ret.append("SHF_OS_NONCONFORMING")
  if 0x200 & byte == 0x200: ret.append("SHF_GROUP")
  if 0x400 & byte == 0x400: ret.append("SHF_TLS")
  if 0xff00000 & byte == 0xff00000: ret.append("SHF_MASKOS")

  return ", ".join(ret)

def parse_pe_flags(byte):
  ret = []

  if 0x10000000 & byte == 0x10000000: ret.append("Shareable")
  if 0x20000000 & byte == 0x20000000: ret.append("Executable")
  if 0x40000000 & byte == 0x40000000: ret.append("Readable")
  if 0x80000000 & byte == 0x80000000: ret.append("Writeable")
  if 0x01000000 & byte == 0x01000000: ret.append("Contain extended relocation")
  if 0x02000000 & byte == 0x02000000: ret.append("Discardable as needed")
  if 0x04000000 & byte == 0x04000000: ret.append("Cant be cached")
  if 0x00001000 & byte == 0x00001000: ret.append("Contain COMDAT data")
  if 0x00000200 & byte == 0x00000200: ret.append("Contais comments or other infos")
  if 0x00000800 & byte == 0x00000800: ret.append("Wont become part of the image")
  if 0x00000020 & byte == 0x00000020: ret.append("Contain executable code")
  if 0x00000040 & byte == 0x00000040: ret.append("Contain initialized data")
  if 0x00000080 & byte == 0x00000080: ret.append("Contain uninitialized data")
  if 0x00000008 & byte == 0x00000008: ret.append("Shouldnt be padded to next boundary")

  return ", ".join(ret)

def search_pe(filename, cavesize):
  g = MicrosoftPe.from_file(filename)

  for section in g.sections:
    section_offset = section.pointer_to_raw_data
    infos = parse_pe_flags(section.characteristics)
    search_cave(section.name, section.body, cavesize, section_offset, section.virtual_address, infos)

def search_macho(filename, cavesize):
  g = MachO.from_file(filename)

  for command in g.load_commands:
    if command.type == MachO.LoadCommandType.segment_64:
      for section in command.body.sections:
        if isinstance(section.data, str):
          infos = "init: [{}], max: [{}]".format(parse_macho_flags(command.body.initprot), parse_macho_flags(command.body.maxprot))
          search_cave("{}.{}".format(section.seg_name, section.sect_name), section.data, cavesize, section.offset, section.addr, infos)

def search_elf(filename, cavesize):
  g = Elf.from_file(filename)

  for section in g.header.section_headers:
    infos = parse_sh_flags(section.flags)
    search_cave(section.name, section.body, cavesize, section.offset, section.addr, infos)

def detect_type(filename, cavesize):
  data = open(filename, "rb").read()

  mz    = "MZ"
  elf   = "\x7FELF"
  macho = pack("I", 0xfeedfacf)

  if   data[0x0:0x2] == mz:    search_pe(filename, cavesize)
  elif data[0x0:0x4] == elf:   search_elf(filename, cavesize)
  elif data[0x0:0x4] == macho: search_macho(filename, cavesize)

def search(filename, cavesize):
  print "{}[*]{} Starting cave mining process...{}".format(Bcolors.YELLOW, Bcolors.BOLD, Bcolors.ENDC)
  print

  detect_type(filename, parse_int(cavesize))

  print "{}[*]{} Mining finished.{}".format(Bcolors.YELLOW, Bcolors.BOLD, Bcolors.ENDC)



# This is a generated file! Please edit source .ksy file and use kaitai-struct-compiler to rebuild

from pkg_resources import parse_version
from kaitaistruct import __version__ as ks_version, KaitaiStruct, KaitaiStream, BytesIO
from enum import Enum
import struct


if parse_version(ks_version) < parse_version('0.7'):
    raise Exception("Incompatible Kaitai Struct Python API: 0.7 or later is required, but you have %s" % (ks_version))

class Elf(KaitaiStruct):

    class Endian(Enum):
        le = 1
        be = 2

    class ShType(Enum):
        null_type = 0
        progbits = 1
        symtab = 2
        strtab = 3
        rela = 4
        hash = 5
        dynamic = 6
        note = 7
        nobits = 8
        rel = 9
        shlib = 10
        dynsym = 11
        init_array = 14
        fini_array = 15
        preinit_array = 16
        group = 17
        symtab_shndx = 18
        sunw_capchain = 1879048175
        sunw_capinfo = 1879048176
        sunw_symsort = 1879048177
        sunw_tlssort = 1879048178
        sunw_ldynsym = 1879048179
        sunw_dof = 1879048180
        sunw_cap = 1879048181
        sunw_signature = 1879048182
        sunw_annotate = 1879048183
        sunw_debugstr = 1879048184
        sunw_debug = 1879048185
        sunw_move = 1879048186
        sunw_comdat = 1879048187
        sunw_syminfo = 1879048188
        sunw_verdef = 1879048189
        sunw_verneed = 1879048190
        sunw_versym = 1879048191
        sparc_gotdata = 1879048192
        amd64_unwind = 1879048193

    class OsAbi(Enum):
        system_v = 0
        hp_ux = 1
        netbsd = 2
        gnu = 3
        solaris = 6
        aix = 7
        irix = 8
        freebsd = 9
        tru64 = 10
        modesto = 11
        openbsd = 12
        openvms = 13
        nsk = 14
        aros = 15
        fenixos = 16
        cloudabi = 17
        openvos = 18

    class Machine(Enum):
        not_set = 0
        sparc = 2
        x86 = 3
        mips = 8
        powerpc = 20
        arm = 40
        superh = 42
        ia_64 = 50
        x86_64 = 62
        aarch64 = 183

    class Bits(Enum):
        b32 = 1
        b64 = 2

    class PhType(Enum):
        null_type = 0
        load = 1
        dynamic = 2
        interp = 3
        note = 4
        shlib = 5
        phdr = 6
        tls = 7
        gnu_eh_frame = 1685382480
        gnu_stack = 1685382481
        hios = 1879048191

    class ObjType(Enum):
        relocatable = 1
        executable = 2
        shared = 3
        core = 4
    def __init__(self, _io, _parent=None, _root=None):
        self._io = _io
        self._parent = _parent
        self._root = _root if _root else self
        self._read()

    def _read(self):
        self.magic = self._io.ensure_fixed_contents(struct.pack('4b', 127, 69, 76, 70))
        self.bits = self._root.Bits(self._io.read_u1())
        self.endian = self._root.Endian(self._io.read_u1())
        self.ei_version = self._io.read_u1()
        self.abi = self._root.OsAbi(self._io.read_u1())
        self.abi_version = self._io.read_u1()
        self.pad = self._io.read_bytes(7)
        self.header = self._root.EndianElf(self._io, self, self._root)

    class EndianElf(KaitaiStruct):
        def __init__(self, _io, _parent=None, _root=None):
            self._io = _io
            self._parent = _parent
            self._root = _root if _root else self
            _on = self._root.endian
            if _on == self._root.Endian.le:
                self._is_le = True
            elif _on == self._root.Endian.be:
                self._is_le = False

            if self._is_le == True:
                self._read_le()
            elif self._is_le == False:
                self._read_be()
            else:
                raise Exception("Unable to decide endianness")

        def _read_le(self):
            self.e_type = self._root.ObjType(self._io.read_u2le())
            self.machine = self._root.Machine(self._io.read_u2le())
            self.e_version = self._io.read_u4le()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.entry_point = self._io.read_u4le()
            elif _on == self._root.Bits.b64:
                self.entry_point = self._io.read_u8le()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.program_header_offset = self._io.read_u4le()
            elif _on == self._root.Bits.b64:
                self.program_header_offset = self._io.read_u8le()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.section_header_offset = self._io.read_u4le()
            elif _on == self._root.Bits.b64:
                self.section_header_offset = self._io.read_u8le()
            self.flags = self._io.read_bytes(4)
            self.e_ehsize = self._io.read_u2le()
            self.program_header_entry_size = self._io.read_u2le()
            self.qty_program_header = self._io.read_u2le()
            self.section_header_entry_size = self._io.read_u2le()
            self.qty_section_header = self._io.read_u2le()
            self.section_names_idx = self._io.read_u2le()

        def _read_be(self):
            self.e_type = self._root.ObjType(self._io.read_u2be())
            self.machine = self._root.Machine(self._io.read_u2be())
            self.e_version = self._io.read_u4be()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.entry_point = self._io.read_u4be()
            elif _on == self._root.Bits.b64:
                self.entry_point = self._io.read_u8be()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.program_header_offset = self._io.read_u4be()
            elif _on == self._root.Bits.b64:
                self.program_header_offset = self._io.read_u8be()
            _on = self._root.bits
            if _on == self._root.Bits.b32:
                self.section_header_offset = self._io.read_u4be()
            elif _on == self._root.Bits.b64:
                self.section_header_offset = self._io.read_u8be()
            self.flags = self._io.read_bytes(4)
            self.e_ehsize = self._io.read_u2be()
            self.program_header_entry_size = self._io.read_u2be()
            self.qty_program_header = self._io.read_u2be()
            self.section_header_entry_size = self._io.read_u2be()
            self.qty_section_header = self._io.read_u2be()
            self.section_names_idx = self._io.read_u2be()

        class ProgramHeader(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None, _is_le=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._is_le = _is_le

                if self._is_le == True:
                    self._read_le()
                elif self._is_le == False:
                    self._read_be()
                else:
                    raise Exception("Unable to decide endianness")

            def _read_le(self):
                self.type = self._root.PhType(self._io.read_u4le())
                if self._root.bits == self._root.Bits.b64:
                    self.flags64 = self._io.read_u4le()

                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.offset = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.offset = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.vaddr = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.vaddr = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.paddr = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.paddr = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.filesz = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.filesz = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.memsz = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.memsz = self._io.read_u8le()
                if self._root.bits == self._root.Bits.b32:
                    self.flags32 = self._io.read_u4le()

                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.align = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.align = self._io.read_u8le()

            def _read_be(self):
                self.type = self._root.PhType(self._io.read_u4be())
                if self._root.bits == self._root.Bits.b64:
                    self.flags64 = self._io.read_u4be()

                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.offset = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.offset = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.vaddr = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.vaddr = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.paddr = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.paddr = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.filesz = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.filesz = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.memsz = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.memsz = self._io.read_u8be()
                if self._root.bits == self._root.Bits.b32:
                    self.flags32 = self._io.read_u4be()

                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.align = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.align = self._io.read_u8be()


        class SectionHeader(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None, _is_le=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._is_le = _is_le

                if self._is_le == True:
                    self._read_le()
                elif self._is_le == False:
                    self._read_be()
                else:
                    raise Exception("Unable to decide endianness")

            def _read_le(self):
                self.name_offset = self._io.read_u4le()
                self.type = self._root.ShType(self._io.read_u4le())
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.flags = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.flags = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.addr = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.addr = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.offset = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.offset = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.size = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.size = self._io.read_u8le()
                self.linked_section_idx = self._io.read_u4le()
                self.info = self._io.read_bytes(4)
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.align = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.align = self._io.read_u8le()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.entry_size = self._io.read_u4le()
                elif _on == self._root.Bits.b64:
                    self.entry_size = self._io.read_u8le()

            def _read_be(self):
                self.name_offset = self._io.read_u4be()
                self.type = self._root.ShType(self._io.read_u4be())
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.flags = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.flags = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.addr = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.addr = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.offset = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.offset = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.size = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.size = self._io.read_u8be()
                self.linked_section_idx = self._io.read_u4be()
                self.info = self._io.read_bytes(4)
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.align = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.align = self._io.read_u8be()
                _on = self._root.bits
                if _on == self._root.Bits.b32:
                    self.entry_size = self._io.read_u4be()
                elif _on == self._root.Bits.b64:
                    self.entry_size = self._io.read_u8be()

            @property
            def body(self):
                if hasattr(self, '_m_body'):
                    return self._m_body if hasattr(self, '_m_body') else None

                io = self._root._io
                _pos = io.pos()
                io.seek(self.offset)
                if self._is_le:
                    self._m_body = io.read_bytes(self.size)
                else:
                    self._m_body = io.read_bytes(self.size)
                io.seek(_pos)
                return self._m_body if hasattr(self, '_m_body') else None

            @property
            def name(self):
                if hasattr(self, '_m_name'):
                    return self._m_name if hasattr(self, '_m_name') else None

                io = self._root.header.strings._io
                _pos = io.pos()
                io.seek(self.name_offset)
                if self._is_le:
                    self._m_name = (io.read_bytes_term(0, False, True, True)).decode(u"ASCII")
                else:
                    self._m_name = (io.read_bytes_term(0, False, True, True)).decode(u"ASCII")
                io.seek(_pos)
                return self._m_name if hasattr(self, '_m_name') else None


        class StringsStruct(KaitaiStruct):
            def __init__(self, _io, _parent=None, _root=None, _is_le=None):
                self._io = _io
                self._parent = _parent
                self._root = _root if _root else self
                self._is_le = _is_le

                if self._is_le == True:
                    self._read_le()
                elif self._is_le == False:
                    self._read_be()
                else:
                    raise Exception("Unable to decide endianness")

            def _read_le(self):
                self.entries = []
                while not self._io.is_eof():
                    self.entries.append((self._io.read_bytes_term(0, False, True, True)).decode(u"ASCII"))


            def _read_be(self):
                self.entries = []
                while not self._io.is_eof():
                    self.entries.append((self._io.read_bytes_term(0, False, True, True)).decode(u"ASCII"))



        @property
        def program_headers(self):
            if hasattr(self, '_m_program_headers'):
                return self._m_program_headers if hasattr(self, '_m_program_headers') else None

            _pos = self._io.pos()
            self._io.seek(self.program_header_offset)
            if self._is_le:
                self._raw__m_program_headers = [None] * (self.qty_program_header)
                self._m_program_headers = [None] * (self.qty_program_header)
                for i in range(self.qty_program_header):
                    self._raw__m_program_headers[i] = self._io.read_bytes(self.program_header_entry_size)
                    io = KaitaiStream(BytesIO(self._raw__m_program_headers[i]))
                    self._m_program_headers[i] = self._root.EndianElf.ProgramHeader(io, self, self._root, self._is_le)

            else:
                self._raw__m_program_headers = [None] * (self.qty_program_header)
                self._m_program_headers = [None] * (self.qty_program_header)
                for i in range(self.qty_program_header):
                    self._raw__m_program_headers[i] = self._io.read_bytes(self.program_header_entry_size)
                    io = KaitaiStream(BytesIO(self._raw__m_program_headers[i]))
                    self._m_program_headers[i] = self._root.EndianElf.ProgramHeader(io, self, self._root, self._is_le)

            self._io.seek(_pos)
            return self._m_program_headers if hasattr(self, '_m_program_headers') else None

        @property
        def section_headers(self):
            if hasattr(self, '_m_section_headers'):
                return self._m_section_headers if hasattr(self, '_m_section_headers') else None

            _pos = self._io.pos()
            self._io.seek(self.section_header_offset)
            if self._is_le:
                self._raw__m_section_headers = [None] * (self.qty_section_header)
                self._m_section_headers = [None] * (self.qty_section_header)
                for i in range(self.qty_section_header):
                    self._raw__m_section_headers[i] = self._io.read_bytes(self.section_header_entry_size)
                    io = KaitaiStream(BytesIO(self._raw__m_section_headers[i]))
                    self._m_section_headers[i] = self._root.EndianElf.SectionHeader(io, self, self._root, self._is_le)

            else:
                self._raw__m_section_headers = [None] * (self.qty_section_header)
                self._m_section_headers = [None] * (self.qty_section_header)
                for i in range(self.qty_section_header):
                    self._raw__m_section_headers[i] = self._io.read_bytes(self.section_header_entry_size)
                    io = KaitaiStream(BytesIO(self._raw__m_section_headers[i]))
                    self._m_section_headers[i] = self._root.EndianElf.SectionHeader(io, self, self._root, self._is_le)

            self._io.seek(_pos)
            return self._m_section_headers if hasattr(self, '_m_section_headers') else None

        @property
        def strings(self):
            if hasattr(self, '_m_strings'):
                return self._m_strings if hasattr(self, '_m_strings') else None

            _pos = self._io.pos()
            self._io.seek(self.section_headers[self.section_names_idx].offset)
            if self._is_le:
                self._raw__m_strings = self._io.read_bytes(self.section_headers[self.section_names_idx].size)
                io = KaitaiStream(BytesIO(self._raw__m_strings))
                self._m_strings = self._root.EndianElf.StringsStruct(io, self, self._root, self._is_le)
            else:
                self._raw__m_strings = self._io.read_bytes(self.section_headers[self.section_names_idx].size)
                io = KaitaiStream(BytesIO(self._raw__m_strings))
                self._m_strings = self._root.EndianElf.StringsStruct(io, self, self._root, self._is_le)
            self._io.seek(_pos)
            return self._m_strings if hasattr(self, '_m_strings') else None






from setuptools import setup
setup(
  name = 'cave_miner',
  packages = ['cave_miner', 'cave_miner.formats'],
  entry_points={
    'console_scripts': [
      'cave_miner=cave_miner.main:main',
    ],
  },
  install_requires = ['docopt==0.6.2', 'kaitaistruct==0.7'],
  version = '1.5',
  description = 'Search for code cave in all binaries',
  author = 'DENIAU Antonin',
  author_email = 'antonin.deniau@protonmail.com',
  url = 'https://github.com/Antonin-Deniau/cave_miner',
  keywords = ['code', 'cave', 'codecave', 'macho', 'mach-o', 'elf', 'pe'],
  classifiers = [],
)

